@@ -7,6 +7,7 @@
 # Import statements:
 import json
 import os
+import logging
 from abc import abstractmethod
 
 import matplotlib.pyplot as plt
@@ -31,7 +32,8 @@ class NewsSegmentation:
                  lcm: tuple = (0.614,),
                  ref: int = 0,
                  cache_file: str = '',
-                 dump: bool = True):
+                 dump: bool = True,
+                 transformation_path: str = ''):
         """
         Virtual class: 3 methods must be overriden
         :param tdm: Penalty factor (betta) for Temporal Distance Manager
@@ -40,6 +42,11 @@ class NewsSegmentation:
                  [1]: Weight for GPA.
             [1]: SDM algorithm parameters.
         :param lcm: Algorithm parameters for Later Correlation Manager.
+        :param ref: Reference news for the evaluation.
+
+        :param cache_file: Path to the cache file.
+        :param dump: Dump the cache file. If True, it dumps the new information into the cache file.
+        :param transformation_path: Path to the database transformation file. If it is not provided, it will be removed.
         """
         # Parameters for each module of the architecture.
         self.parameters = {"betta_tdm": tdm, "gpa": gpa, "sdm": sdm, "lcm": lcm}
@@ -67,7 +74,7 @@ class NewsSegmentation:
                         for key, item in _cache.items():
                             self._cache[key] = np.array(item)
                 except Exception as ex:
-                    print(ex)
+                    logging.error(ex)
                     self._cache_file = False
         # Steps for in the architecture.
         if '.txt' in news_path:
@@ -105,10 +112,25 @@ class NewsSegmentation:
             self._dump_cache()
 
         if os.path.exists(temporalfile):
-            os.remove(temporalfile)
+            if transformation_path == '':
+                os.remove(temporalfile)
+            else:
+                os.rename(temporalfile, transformation_path)
 
     def __database_transformation(self, path: str):
-        return self._database_transformation(path, temporalfile)
+        try:
+            _db_trans_ = self._database_transformation(path, temporalfile)
+        except Exception as ex:
+            raise RuntimeError(f'DBT: Exception from the "Database transformation" function: {ex}')
+        if _db_trans_:
+            if os.path.exists(_db_trans_):
+                if isinstance(_db_trans_, str):
+                    return _db_trans_
+                else:
+                    raise ValueError('DBT: The return value of the "Database transformation" is not a string type.')
+            else:
+                raise ValueError('DBT: The return value of the "Database transformation" is not an existing path.')
+        raise ValueError('DBT: The return value of the "Database transformation" function must be a temporary path.')
 
     @staticmethod
     def __reader(path: str):
@@ -135,16 +157,50 @@ class NewsSegmentation:
                 s[index] = sentence[:-1]
             if sentence[0] == ' ':
                 s[index] = sentence[1:]
+
+        # Check t vectors if they are empty.
+        if isinstance(t, list):
+            t = np.array(t)
+        if t.size == 0:
+            t = np.array([1. for _ in s])
+
         return p, s, t
 
-    def __specific_language_model(self, s: list[str]) -> np.array:
+    def __specific_language_model(self, s: list[str]) -> np.ndarray:
         # Check the cache.
         sx = []
+        all_cached = True
         for sentence in s:
             if sentence not in self._cache:
                 sx.append(sentence)
+                all_cached = False
         # Get the embeddings.
-        embeddings_ = self._specific_language_model(sx)
+        try:
+            embeddings_ = self._specific_language_model(sx)
+        except Exception as ex:
+            raise RuntimeError(f'SLM: Exception from the "SLM" function: {ex}')
+        if not all_cached:
+            if embeddings_.any():
+                if isinstance(embeddings_, np.ndarray):
+                    if len(embeddings_.shape) == 2:
+                        if embeddings_.dtype != np.float16 and embeddings_.dtype != np.float32 and \
+                                embeddings_.dtype != np.float64 and embeddings_.dtype != np.int8 and \
+                                embeddings_.dtype != np.int16 and embeddings_.dtype != np.int32 and \
+                                embeddings_.dtype != np.int64:
+                            raise ValueError(
+                                f'SLM: The return value of the "SLM" function must be a embedding vector '
+                                f'(numpy.ndarray) containing real numbers (np.intXX or np.floatXX) not '
+                                f'{embeddings_.dtype}...')
+                    else:
+                        raise ValueError(f'SLM: The return value of the "SLM" function must be a embedding vector '
+                                         f'(numpy array) with 2 dimensions: (number_of_sentences , embedding_length); '
+                                         f'not {len(embeddings_.shape)} dimensions.')
+                else:
+                    raise ValueError(f'SLM: The return value of the "SLM" function must be a embedding vector '
+                                     f'(numpy array) not a {type(embeddings_)}.')
+            else:
+                raise ValueError('SLM: The return value of the "SLM" function must be a embedding vector '
+                                 '(numpy array).')
         # Store the embeddings in cache.
         for place, sentence in enumerate(sx):
             self._cache[sentence] = embeddings_[place]
@@ -179,9 +235,41 @@ class NewsSegmentation:
                     if self.parameters['gpa'][0] > 0 else element
                 rp[nr][nc] = element + self.parameters['gpa'][1] * (gpa - element)
         # SDM algorithm:
-        self._directives = self._spatial_manager(rp, self.parameters['sdm'])
-        if not self._directives:
-            self._directives = list(range(1, len(rp)))
+        try:
+            _directives = self._spatial_manager(rp, self.parameters['sdm'])
+        except Exception as ex:
+            raise RuntimeError(f'SDM: Exception raised from the "SDM" function: {ex}')
+        if not _directives:
+            _directives = list(range(1, len(rp)))
+
+        if isinstance(_directives, list):
+            try:
+                _directives.sort()
+                while _directives[0] <= 0:
+                    logging.info(f'SDM: popping a wrong element of the directives: {_directives.pop(0)}, because '
+                                 f'it is less than 1.')
+                while _directives[-1] > len(rp):
+                    logging.info(f'SDM: popping a wrong element of the directives: {_directives.pop(-1)}, because '
+                                 f'it is greater than the length of the matrix.')
+                if len(rp) not in _directives:
+                    logging.info(f'SDM: adding a missing element of the directives: {len(rp)}, because '
+                                 f'it is always required.')
+                    _directives.append(len(rp))
+                if len(set(_directives)) != len(_directives):
+                    raise ValueError(f'SDM: The return value of the "SDM" function must be a directive list, '
+                                     f'with no repeated values inside. The list contains '
+                                     f'{len(_directives) - len(set(_directives))} repeated values. Fix it and retry.')
+            except Exception as ex:
+                raise ValueError(f'SDM: The return value of the "SDM" function must be a directive list, '
+                                 f'it must be sorted, and belong to the (0, len(matrix)] interval, containing the'
+                                 f'length of the matrix in the last place. If the module detects that this is wrong,'
+                                 f'we try to fix it. However, the following exception raised when trying to fix it: '
+                                 f'{ex}.')
+        else:
+            raise ValueError(f'SDM: The return value of the "SDM" function must be a directive list, '
+                             f'not a {type(_directives)}.')
+
+        self._directives = _directives
         # Merge components:
         s2 = []
         t2 = []
@@ -203,8 +291,43 @@ class NewsSegmentation:
         return rp, s2, np.array(t2)
 
     def __later_correlation_manager(self, s: list[str], t: (list[float], np.ndarray)):
-        r, s, t = self._later_correlation_manager(self.__specific_language_model, s, t, self.parameters['lcm'])
-        return np.array(r), s, np.array(t)
+        try:
+            r, s, t = self._later_correlation_manager(self.__specific_language_model, s, t, self.parameters['lcm'])
+        except Exception as ex:
+            raise RuntimeError(f'LCM: An exception raised from the "LCM" function: {ex}')
+        if len(s) == len(t) == len(r):
+            if isinstance(s, list):
+                if isinstance(s[0], str):
+                    if isinstance(t, (list, np.ndarray)):
+                        if isinstance(r, (np.ndarray, list)):
+                            if len(r) == len(r[0]):
+                                try:
+                                    return np.array(r), s, np.array(t)
+                                except Exception as ex:
+                                    raise RuntimeError(f'LCM: An error ocurred in the LCM while converting the output '
+                                                       f'values to np.arrays: {ex}')
+                            else:
+                                raise ValueError(
+                                    f'LCM: The size of the returned matrix must be squared. '
+                                    f'Current shape: {len(r)} x {len(r[0])}')
+                        else:
+                            raise ValueError(
+                                f'LCM: The first return value of the LCM must be a correlation matrix, not {type(r)}.')
+                    else:
+                        raise ValueError(
+                            f'LCM: The third return value of the LCM must be a list or np.ndarray, not '
+                            f'{type(t)}.')
+                else:
+                    raise ValueError(
+                        f'LCM: The second return value of the LCM must be a list of strings, not a list of '
+                        f'{type(s[0])}.')
+            else:
+                raise ValueError(
+                    f'LCM: The second return value of the LCM must be a list, not {type(s)}.')
+        else:
+            raise ValueError(f'LCM: The dimensions of the returned matrix, the sentence vector and the '
+                             f'temporal distance vector must be equal: '
+                             f'matrix:{len(s)} == temporal:{len(t)} == sentence:{len(s)}')
 
     def __treefication(self):
         initial_s = self.s[0]
@@ -235,7 +358,7 @@ class NewsSegmentation:
                 if nc > nr:
                     cp += element ** 2
         cp /= len(cembedding) * (len(cembedding) - 1) if len(cembedding) > 1 else 1
-        return np.float(2 * cp)
+        return np.float32(2 * cp)
 
     @staticmethod
     @abstractmethod
@@ -249,12 +372,12 @@ class NewsSegmentation:
 
     @staticmethod
     @abstractmethod
-    def _specific_language_model(s: list[str]) -> np.array:
+    def _specific_language_model(s: list[str]) -> np.ndarray:
         pass
 
     @staticmethod
     @abstractmethod
-    def _database_transformation(path: str, output: str):
+    def _database_transformation(path: str, output: str) -> str:
         pass
 
     def __repr__(self):
@@ -289,7 +412,7 @@ class NewsSegmentation:
 
     def plotmtx(self, *args, color='orange'):
         if color not in ['orange', 'black']:
-            print(f'Warning: Color: {color} not defined. Orange chosen.')
+            logging.warning(f'Warning: Color: {color} not defined. Orange chosen.')
             color = 'orange'
         if len(args) == 0:
             args_ = (0, 1, 2, 3)
@@ -492,7 +615,7 @@ class NewsSegmentation:
             subfigs[2].imshow(mtwd)
             subfigs[2].set_title('Segmentation boundaries.')
             subfigs[2].set_xlabel('Sentence index')
-            subfigs[2].set_yticks(np.array([0, 1]), labels=['Reference', 'Performed'])
+            subfigs[2].set_yticks(np.array([0, 1]), labels=['Performed', 'Reference'])
 
             mtreep = np.zeros((max(len(self.News), len(reference_trees)), len(original), 3), dtype=np.uint8)
             for ns, sentence in enumerate(original):
@@ -548,7 +671,7 @@ class NewsSegmentation:
                         _cache[key] = item.tolist()
                     json.dump(_cache, file)
             except Exception as ex:
-                print(ex)
+                logging.error(ex)
 # - x - x - x - x - x - x - x - x - x - x - x - x - x - x - #
 #                        END OF FILE                        #
 # - x - x - x - x - x - x - x - x - x - x - x - x - x - x - #
